/*  signup.css
 *
 *  Description:
 *  With base elements out of the way, this sets all custom styling for the extension.
 *
 *  Contents:
 *  Global
 *  Core detail
 *  Core detail - storyboard
 *  Buttons
 *  Overflow mode
 *  Language overrides
 */

/*=Global
--------------------------------------------------------------------------------------- */
.pkt_ext_containersignup {
    background-color: #ebebeb;
    color: #333;
    display: block;
    font-size: 16px;
    font-family: "FiraSans", "Helvetica Neue", Helvetica, Arial, sans-serif;
    margin: 0;
    padding: 0;
    position: relative;
    text-align: center;
}
.pkt_ext_containersignup_inactive {
    animation: pkt_ext_hide 0.3s ease-out;
    opacity: 0;
    visibility: hidden;
}
.pkt_ext_cf:after {
    content: " ";
    display:table;
    clear:both;
}
@keyframes pkt_ext_hide {
    0% {
        opacity: 1;
        visibility: visible;
    }
    99% {
        opacity: 0;
        visibility: visible;
    }
    100% {
        opacity: 0;
        visibility: hidden;
    }
}

/*=Core detail
--------------------------------------------------------------------------------------- */
.pkt_ext_containersignup p {
    font-size: 14px;
    color: #333;
    font-family: "FiraSans", "Helvetica Neue", Helvetica, Arial, sans-serif;
    line-height: 1.3;
    margin: 0 auto 1.5em;
    max-width: 260px;
}
.pkt_ext_containersignup a { 
    color: #4c8fd0;
}
.pkt_ext_containersignup a:hover {
    color: #3076b9;
}
.pkt_ext_containersignup .pkt_ext_introdetail {
    background-color: #fbfbfb;
    border: 1px solid #c1c1c1;
    border-width: 0 0 1px;
}
.pkt_ext_containersignup .pkt_ext_logo {
    background: url(../img/pocketlogo@1x.png) center bottom no-repeat;
    display: block;
    height: 32px;
    margin: 0 auto 15px;
    padding-top: 25px;
    position: relative;
    text-indent: -9999px;
    width: 123px;
}
@media (min-resolution: 1.1dppx) {
    .pkt_ext_containersignup .pkt_ext_logo {
        background-image: url(../img/pocketlogo@2x.png);
        background-size: 123px 32px;
    }
}
.pkt_ext_containersignup .pkt_ext_introimg {
    background: url(../img/pocketsignup_hero@1x.png) center center no-repeat;
    display: block;
    height: 125px;
    margin: 0 auto;
    position: relative;
    text-indent: -9999px;
    width: 255px;
}
@media (min-resolution: 1.1dppx) {
    .pkt_ext_containersignup .pkt_ext_introimg {
        background-image: url(../img/pocketsignup_hero@2x.png);
        background-size: 255px 125px;
    }
}
.pkt_ext_containersignup .pkt_ext_tagline {
    margin-bottom: 0.5em;
}
.pkt_ext_containersignup .pkt_ext_learnmore {
    font-size: 12px;
}
.pkt_ext_containersignup .pkt_ext_learnmoreinactive {
    visibility: hidden;
}
.pkt_ext_signupdetail h4 {
    font-size: 12px;
    font-weight: normal;
}
.pkt_ext_signupdetail .btn-container {
    position: relative;
    margin-bottom: 0.8em;
}
.pkt_ext_containersignup .ff_signuphelp {
    background: url(../img/signup_help@1x.png) center center no-repeat;
    display: block;
    height: 18px;
    margin-top: -9px;
    right: -15px;
    position: absolute;
    text-indent: -9999px;
    width: 18px;
    top: 50%;
}
@media (min-resolution: 1.1dppx) {
    .pkt_ext_containersignup .ff_signuphelp {
        background-image: url(../img/signup_help@2x.png);
        background-size: 18px 18px;
    }
}
.pkt_ext_containersignup .alreadyhave {
    font-size: 12px;
    max-width: 320px;
    margin-top: 15px;
}

/*=Core detail - storyboard
--------------------------------------------------------------------------------------- */
.pkt_ext_introstory {
    align-items: center;
    display: flex;
    padding: 20px;
}
.pkt_ext_introstory:after {
    clear: both;
    content: "";
    display: table;
}
.pkt_ext_introstory p {
    margin-bottom: 0;
    text-align: left;
}
.pkt_ext_introstoryone {
    padding: 20px 18px 15px 20px;
}
.pkt_ext_introstorytwo {
    padding: 3px 0 0 20px;
}
.pkt_ext_introstorytwo .pkt_ext_tagline {
    margin-bottom: 1.5em;
}
.pkt_ext_introstory_text {
    flex: 1;
}
.pkt_ext_introstoryone_img,
.pkt_ext_introstorytwo_img {
    display: block;
    overflow: hidden;
    position: relative;
    text-indent: -999px;
}
.pkt_ext_introstoryone_img {
    background: url(../img/pocketsignup_button@1x.png) center right no-repeat;
    height: 82px;
    padding: 0 0 0 0.7em;
    width: 82px;
}
@media (min-resolution: 1.1dppx) {
    .pkt_ext_introstoryone_img {
        background-image: url(../img/pocketsignup_button@2x.png);
        background-size: 82px 82px;
    }
}
.pkt_ext_introstorytwo_img {
    background: url(../img/pocketsignup_devices@1x.png) bottom right no-repeat;
    height: 110px;
    padding: 1em 0 0 0.7em;
    width: 124px;
}
@media (min-resolution: 1.1dppx) {
    .pkt_ext_introstorytwo_img {
        background-image: url(../img/pocketsignup_devices@2x.png);
        background-size: 124px 110px;
    }
}
.pkt_ext_introstorydivider {
    border-top: 1px solid #c1c1c1;
    height: 1px;
    margin: 0 auto;
    width: 125px;
}

/*=Buttons
--------------------------------------------------------------------------------------- */
.pkt_ext_containersignup .btn {
    background-color: #0096dd;
    border: 1px solid #0095dd;
    border-radius: 2px;
    color: #fff;
    display: inline-block;
    font-family: "FiraSans", "Helvetica Neue", Helvetica, Arial, sans-serif;
    font-size: 16px;
    font-weight: normal;
    line-height: 1;
    margin: 0;
    padding: 11px 45px;
    text-align: center;
    text-decoration: none;
    text-shadow: 0 -1px 0 rgba(142,4,17,0.5);
    transition: background-color 0.1s linear;
    width: auto;
}
.pkt_ext_containersignup .btn-secondary {
    background-color: #fbfbfb;
    border-color: #c1c1c1;
    color: #444;
    text-shadow: 0 1px 0 rgba(255,255,255,0.5);
}
.pkt_ext_containersignup .btn-small {
    padding: 6px 20px;
}
.pkt_ext_containersignup .btn-mini {
    font-size: 14px;
    padding: 5px 15px 4px;
}
.pkt_ext_containersignup .btn:hover {
    background-color: #008acb;
    color: #fff;
    text-decoration: none;
}
.pkt_ext_containersignup .btn-secondary:hover,
.pkt_ext_containersignup .btn-important:hover {
    background-color: #f6f6f6;
    color: #222;
}
.pkt_ext_containersignup .btn-disabled {
    background-image: none;
    color: #ccc;
    color: rgba(255,255,255,0.6);
    cursor: default;
    opacity: 0.9;
}
.pkt_ext_containersignup .signup-btn-firefox,
.pkt_ext_containersignup .signup-btn-email,
.pkt_ext_containersignup .signupinterim-btn-login,
.pkt_ext_containersignup .signupinterim-btn-signup,
.pkt_ext_containersignup .forgot-btn-submit,
.pkt_ext_containersignup .forgotreset-btn-change {
    min-width: 12.125em;
    padding: 0.8em 1.1875em;
    box-sizing: content-box;
}
.pkt_ext_containersignup .signup-btn-email {
    position: relative;
    z-index: 10;
}
.pkt_ext_containersignup .signup-btn-firefox {
    min-width: 14.5em;
    position: relative;
    padding: 0;
}
.pkt_ext_containersignup .signup-btn-firefox .logo {
    background: url(../img/signup_firefoxlogo@1x.png) center center no-repeat;
    height: 2.6em;
    left: 10px;
    margin: 0;
    padding: 0;
    width: 22px;
    position: absolute;
}
@media (min-resolution: 1.1dppx) {
    .pkt_ext_containersignup .signup-btn-firefox .logo {
        background-image: url(../img/signup_firefoxlogo@2x.png);
        background-size: 22px 22px;
    }
}
.pkt_ext_containersignup .forgotreset-btn-change {
    margin-bottom: 2em;
}
.pkt_ext_containersignup .signup-btn-firefox .text {
    display: inline-block;
    padding: 0.8em 1.625em;
    position: relative;
    text-shadow: none;
    white-space: nowrap;
}
.pkt_ext_containersignup .signup-btn-firefox .text {
    color: #fff;
}
.pkt_ext_containersignup .btn-disabled .text {
    color: #ccc;
    color: rgba(255,255,255,0.6);
}

/*=Overflow mode
--------------------------------------------------------------------------------------- */
.pkt_ext_signup_overflow .pkt_ext_tagline {
    margin-bottom: 1em;
    padding: 0 1em;
}
.pkt_ext_signup_overflow .pkt_ext_introimg {
    background-size: 200px 98px;
    height: 98px;
    width: 200px;
}
.pkt_ext_signup_overflow .signup-btn-firefox,
.pkt_ext_signup_overflow .signup-btn-email {
    font-size: 14px;
    min-width: 12.6em;
    padding-left: 0.75em;
    padding-right: 0.75em;
}
.pkt_ext_signup_overflow .signup-btn-firefox .text {
    padding-left: 0;
    padding-right: 0;
}

/*=Language overrides
--------------------------------------------------------------------------------------- */
.pkt_ext_signup_de .pkt_ext_introstoryone_img {
    margin-right: -5px;
    padding-left: 0;
}
.pkt_ext_signup_de .pkt_ext_introstorytwo .pkt_ext_tagline,
.pkt_ext_signup_es .pkt_ext_introstorytwo .pkt_ext_tagline,
.pkt_ext_signup_ja .pkt_ext_introstorytwo .pkt_ext_tagline,
.pkt_ext_signup_ru .pkt_ext_introstorytwo .pkt_ext_tagline {
    margin-bottom: 0.5em;
}
.pkt_ext_signup_de .signup-btn-firefox .text,
.pkt_ext_signup_de .signup-btn-email,
.pkt_ext_signup_es .pkt_ext_signupdetail_hero .signup-btn-firefox .text,
.pkt_ext_signup_es .pkt_ext_signupdetail_hero .signup-btn-email,
.pkt_ext_signup_ja .signup-btn-firefox .text,
.pkt_ext_signup_ja .signup-btn-email,
.pkt_ext_signup_ru .signup-btn-firefox .text,
.pkt_ext_signup_ru .signup-btn-email {
    font-size: 15px;
}
.pkt_ext_signup_ja .signup-btn-firefox .text,
.pkt_ext_signup_ru .signup-btn-firefox .text {
    left: 15px;
}
.pkt_ext_signup_de .signup-btn-firefox .logo,
.pkt_ext_signup_es .pkt_ext_signupdetail_hero .signup-btn-firefox .logo,
.pkt_ext_signup_ja .signup-btn-firefox .logo,
.pkt_ext_signup_ru .signup-btn-firefox .logo {
    height: 2.4em;
}
@media (min-resolution: 1.1dppx) {
    .pkt_ext_signup_de .signup-btn-firefox .logo,
    .pkt_ext_signup_es .pkt_ext_signupdetail_hero .signup-btn-firefox .logo,
    .pkt_ext_signup_ja .signup-btn-firefox .logo,
    .pkt_ext_signup_ru .signup-btn-firefox .logo {
        height: 2.5em;
    }
}
.pkt_ext_signup_de .signup-btn-email,
.pkt_ext_signup_es .pkt_ext_signupdetail_hero .signup-btn-email,
.pkt_ext_signup_ja .signup-btn-email,
.pkt_ext_signup_ru .signup-btn-email {
    min-width: 13em;
    padding: 0.8533em 1.2667em;
}
.pkt_ext_signup_de .pkt_ext_logo,
.pkt_ext_signup_es .pkt_ext_logo,
.pkt_ext_signup_ru .pkt_ext_logo {
    padding-top: 15px;
}
.pkt_ext_signup_de .pkt_ext_introdetailhero .pkt_ext_tagline,
.pkt_ext_signup_es .pkt_ext_introdetailhero .pkt_ext_tagline,
.pkt_ext_signup_ja .pkt_ext_introdetailhero .pkt_ext_tagline,
.pkt_ext_signup_ru .pkt_ext_introdetailhero .pkt_ext_tagline {
    font-size: 13px;
}
.pkt_ext_signup_overflow.pkt_ext_signup_de .signup-btn-firefox .logo,
.pkt_ext_signup_overflow.pkt_ext_signup_es .signup-btn-firefox .logo,
.pkt_ext_signup_overflow.pkt_ext_signup_ja .signup-btn-firefox .logo,
.pkt_ext_signup_overflow.pkt_ext_signup_ru .signup-btn-firefox .logo {
    display: none;
}