/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifdef __cplusplus
extern "C"
#endif
void *__dso_handle;

/* Begin all files as hidden visibility */
#pragma GCC visibility push(hidden)
